if ( !isObject( "dbGame" ) )
{
	new scriptObject( "dbGame" )
	{
		state = "";
	};

	if ( isObject( dbGame.getGroup() ) )
	{
		dbGame.getGroup().remove( dbGame );
	}
}

function dbGame::startRound(%this) {
	if (%this.state !$= "" && %this.state !$= "ended") {
		return;
	}

	messageAll('', "\c5Starting a new round.");

	%this.state = "starting";
	%count = clientGroup.getCount();

	for (%i = 0; %i < %count; %i++) {
		%cl = clientGroup.getObject(%i);

		if (%cl.hasSpawnedOnce) {
			%cl.instantRespawn();
		}
	}

	%this.state = "active";

	%this.countdown(3);
	%this.spawnRocketTick = %this.schedule(3000, "spawnRocketTick");
}

function dbGame::countdown(%this, %time) {
	cancel(%this.countdown);

	if (%time <= 0) {
		return;
	}

	if (%time == 3) %color = "\c0";
	if (%time == 2) %color = "\c3";
	if (%time == 1) %color = "\c2";

	centerPrintAll("\n\n\n<font:consolas:128>" @ %color @ %time, 0.5);
	%this.countdown = %this.schedule(1000, countdown, %time--);
}

function dbGame::endRound(%this, %message) {
	if (%this.state !$= "active") {
		return;
	}

	messageAll('', "\c5" @ ( strLen( %message ) ? %message : "The round ended." ) SPC "\c5Restarting in five seconds.");

	%this.state = "ended";
	%this.schedule(5000, "startRound");
}

function dbGame::checkWin(%this) {
	if (%this.state !$= "active") {
		return;
	}

	%c0 = 0;
	%c1 = 0;

	%count = clientGroup.getCount();

	for (%i = 0; %i < %count; %i++) {
		%cl = clientGroup.getObject(%i);

		if (!strLen(%cl.dbTeam)) {
			continue;
		}

		if (!isObject(%pl = %cl.player) || %pl.getState() $= "Dead") {
			continue;
		}

		%c[%cl.dbTeam]++;
	}

	if (!%c0 && !%c1) {
		%this.endRound("No team won this round.");
	}
	else if (%c0 && !%c1) {
		%this.endRound("<color:FF5500>RED \c5won this round.");
	}
	else if (!%c0 && %c1) {
		%this.endRound("<color:0055FF>BLU \c5won this round.");
	}
}

function dbGame::spawnRocketTick(%this) {
	cancel(%this.spawnRocketTick);

	if (%this.state !$= "active") {
		return;
	}

	if (!isObject(%this.spawnRocket(rocketLauncherProjectile))) {
		%size = getRandom() * 1 + 1;
		%obj = %this.spawnRocket(dbNukeProjectile, 15 / %size);

		if (isObject(%obj)) {
			%obj.setscale(%size SPC %size SPC %size);
		}
	}

	%this.spawnRocketTick = %this.schedule(1500, "spawnRocketTick");
}

function dbGame::spawnRocket(%this, %db, %speed) {
	if (!isObject(%db)) {
		return -1;
	}

	if (strLen(%db.maxActive) && %this.getRocketTypeCount(%db) >= %db.maxActive) {
		return -1;
	}

	if (%speed $= "") {
		%speed = 20;
	}

	%team = getRandom(0, 1);
	%count = dbBrickGroup.getCount();

	for (%i = 0; %i < %count; %i++) {
		%obj = dbBrickGroup.getObject(%i);

		if (!%obj.getDataBlock().isDBSpawner) {
			continue;
		}

		%list = %list @ (strLen(%list) ? "\t" : "") @ %obj;
	}

	if (%list $= "") {
		return -1;
	}

	%spawn = getField(%list, getRandom(0, getFieldCount(%list) - 1));

	if (!isObject(%spawn)) {
		return -1;
	}

	%dir = -1;

	%obj = new Projectile()
	{
		datablock = %db;
		client = dbClient;

		initialPosition = vectorAdd(%spawn.position, "0 0" SPC %dir);
		initialVelocity = "0 0 0";

		team = !%team;
		speed = %speed;
	};

	dbProjectileGroup.add(%obj);
	%obj.homingTarget = %obj.pickHomingTarget();

	if (!isObject(%obj.homingTarget)) {
		%obj.delete();
		return -1;
	}

	return %obj;
}

function dbGame::getRocketTypeCount(%this, %db) {
	if (!isObject(%db)) {
		return 0;
	}

	%db = %db.getID();

	%value = 0;
	%count = dbProjectileGroup.getCount();

	for (%i = 0; %i < %count; %i++) {
		%value += dbProjectileGroup.getObject(%i).getDataBlock() $= %db;
	}

	return %value;
}
